# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    UserCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUser:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Hanzo) -> None:
        user = client.user.create()
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Hanzo) -> None:
        user = client.user.create(
            aliases={},
            allowed_cache_controls=[{}],
            auto_create_key=True,
            blocked=True,
            budget_duration="budget_duration",
            config={},
            duration="duration",
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            spend=0,
            team_id="team_id",
            teams=[{}],
            tpm_limit=0,
            user_alias="user_alias",
            user_email="user_email",
            user_id="user_id",
            user_role="proxy_admin",
        )
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Hanzo) -> None:
        response = client.user.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Hanzo) -> None:
        with client.user.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserCreateResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Hanzo) -> None:
        user = client.user.update()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Hanzo) -> None:
        user = client.user.update(
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            config={},
            duration="duration",
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            password="password",
            permissions={},
            rpm_limit=0,
            spend=0,
            team_id="team_id",
            tpm_limit=0,
            user_email="user_email",
            user_id="user_id",
            user_role="proxy_admin",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Hanzo) -> None:
        response = client.user.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Hanzo) -> None:
        with client.user.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hanzo) -> None:
        user = client.user.list()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: Hanzo) -> None:
        user = client.user.list(
            page=1,
            page_size=1,
            role="role",
            user_ids="user_ids",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hanzo) -> None:
        response = client.user.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hanzo) -> None:
        with client.user.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Hanzo) -> None:
        user = client.user.delete(
            user_ids=["string"],
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: Hanzo) -> None:
        user = client.user.delete(
            user_ids=["string"],
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Hanzo) -> None:
        response = client.user.with_raw_response.delete(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Hanzo) -> None:
        with client.user.with_streaming_response.delete(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_info(self, client: Hanzo) -> None:
        user = client.user.retrieve_info()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_info_with_all_params(self, client: Hanzo) -> None:
        user = client.user.retrieve_info(
            user_id="user_id",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_info(self, client: Hanzo) -> None:
        response = client.user.with_raw_response.retrieve_info()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_info(self, client: Hanzo) -> None:
        with client.user.with_streaming_response.retrieve_info() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUser:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.create()
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.create(
            aliases={},
            allowed_cache_controls=[{}],
            auto_create_key=True,
            blocked=True,
            budget_duration="budget_duration",
            config={},
            duration="duration",
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            spend=0,
            team_id="team_id",
            teams=[{}],
            tpm_limit=0,
            user_alias="user_alias",
            user_email="user_email",
            user_id="user_id",
            user_role="proxy_admin",
        )
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHanzo) -> None:
        response = await async_client.user.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserCreateResponse, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHanzo) -> None:
        async with async_client.user.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserCreateResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.update()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.update(
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            config={},
            duration="duration",
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            password="password",
            permissions={},
            rpm_limit=0,
            spend=0,
            team_id="team_id",
            tpm_limit=0,
            user_email="user_email",
            user_id="user_id",
            user_role="proxy_admin",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHanzo) -> None:
        response = await async_client.user.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHanzo) -> None:
        async with async_client.user.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.list()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.list(
            page=1,
            page_size=1,
            role="role",
            user_ids="user_ids",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHanzo) -> None:
        response = await async_client.user.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHanzo) -> None:
        async with async_client.user.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.delete(
            user_ids=["string"],
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.delete(
            user_ids=["string"],
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHanzo) -> None:
        response = await async_client.user.with_raw_response.delete(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHanzo) -> None:
        async with async_client.user.with_streaming_response.delete(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_info(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.retrieve_info()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_info_with_all_params(self, async_client: AsyncHanzo) -> None:
        user = await async_client.user.retrieve_info(
            user_id="user_id",
        )
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        response = await async_client.user.with_raw_response.retrieve_info()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(object, user, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        async with async_client.user.with_streaming_response.retrieve_info() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(object, user, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["JobCreateParams", "Hyperparameters"]


class JobCreateParams(TypedDict, total=False):
    custom_llm_provider: Required[Literal["openai", "azure", "vertex_ai"]]

    model: Required[str]

    training_file: Required[str]

    hyperparameters: Optional[Hyperparameters]

    integrations: Optional[List[str]]

    seed: Optional[int]

    suffix: Optional[str]

    validation_file: Optional[str]


class Hyperparameters(TypedDict, total=False):
    batch_size: Union[str, int, None]

    learning_rate_multiplier: Union[str, float, None]

    n_epochs: Union[str, int, None]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .info_list_params import InfoListParams as InfoListParams
from .update_full_params import UpdateFullParams as UpdateFullParams
from .update_partial_params import UpdatePartialParams as UpdatePartialParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)

__all__ = [
    "ContentResource",
    "AsyncContentResource",
    "ContentResourceWithRawResponse",
    "AsyncContentResourceWithRawResponse",
    "ContentResourceWithStreamingResponse",
    "AsyncContentResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .lite_llm_user_table import LiteLlmUserTable
from .organization_membership_table import OrganizationMembershipTable

__all__ = ["OrganizationAddMemberResponse"]


class OrganizationAddMemberResponse(BaseModel):
    organization_id: str

    updated_organization_memberships: List[OrganizationMembershipTable]

    updated_users: List[LiteLlmUserTable]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["UtilTokenCounterParams"]


class UtilTokenCounterParams(TypedDict, total=False):
    model: Required[str]

    messages: Optional[Iterable[object]]

    prompt: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from ..._models import BaseModel

__all__ = ["PassThroughGenericEndpoint"]


class PassThroughGenericEndpoint(BaseModel):
    headers: object
    """Key-value pairs of headers to be forwarded with the request.

    You can set any key value pair here and it will be forwarded to your target
    endpoint
    """

    path: str
    """The route to be added to the LiteLLM Proxy Server."""

    target: str
    """The URL to which requests for this path should be forwarded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CredentialCreateParams"]


class CredentialCreateParams(TypedDict, total=False):
    credential_info: Required[object]

    credential_name: Required[str]

    credential_values: Required[object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    SpendListLogsResponse,
    SpendListTagsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpend:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_calculate_spend(self, client: Hanzo) -> None:
        spend = client.spend.calculate_spend()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_calculate_spend_with_all_params(self, client: Hanzo) -> None:
        spend = client.spend.calculate_spend(
            completion_response={},
            messages=[{}],
            model="model",
        )
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_calculate_spend(self, client: Hanzo) -> None:
        response = client.spend.with_raw_response.calculate_spend()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_calculate_spend(self, client: Hanzo) -> None:
        with client.spend.with_streaming_response.calculate_spend() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(object, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list_logs(self, client: Hanzo) -> None:
        spend = client.spend.list_logs()
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_logs_with_all_params(self, client: Hanzo) -> None:
        spend = client.spend.list_logs(
            api_key="api_key",
            end_date="end_date",
            request_id="request_id",
            start_date="start_date",
            user_id="user_id",
        )
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_logs(self, client: Hanzo) -> None:
        response = client.spend.with_raw_response.list_logs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_logs(self, client: Hanzo) -> None:
        with client.spend.with_streaming_response.list_logs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(SpendListLogsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list_tags(self, client: Hanzo) -> None:
        spend = client.spend.list_tags()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_tags_with_all_params(self, client: Hanzo) -> None:
        spend = client.spend.list_tags(
            end_date="end_date",
            start_date="start_date",
        )
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_tags(self, client: Hanzo) -> None:
        response = client.spend.with_raw_response.list_tags()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = response.parse()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_tags(self, client: Hanzo) -> None:
        with client.spend.with_streaming_response.list_tags() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = response.parse()
            assert_matches_type(SpendListTagsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSpend:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_calculate_spend(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.calculate_spend()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_calculate_spend_with_all_params(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.calculate_spend(
            completion_response={},
            messages=[{}],
            model="model",
        )
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_calculate_spend(self, async_client: AsyncHanzo) -> None:
        response = await async_client.spend.with_raw_response.calculate_spend()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(object, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_calculate_spend(self, async_client: AsyncHanzo) -> None:
        async with async_client.spend.with_streaming_response.calculate_spend() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(object, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_logs(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.list_logs()
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_logs_with_all_params(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.list_logs(
            api_key="api_key",
            end_date="end_date",
            request_id="request_id",
            start_date="start_date",
            user_id="user_id",
        )
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_logs(self, async_client: AsyncHanzo) -> None:
        response = await async_client.spend.with_raw_response.list_logs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(SpendListLogsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_logs(self, async_client: AsyncHanzo) -> None:
        async with async_client.spend.with_streaming_response.list_logs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(SpendListLogsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_tags(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.list_tags()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_tags_with_all_params(self, async_client: AsyncHanzo) -> None:
        spend = await async_client.spend.list_tags(
            end_date="end_date",
            start_date="start_date",
        )
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_tags(self, async_client: AsyncHanzo) -> None:
        response = await async_client.spend.with_raw_response.list_tags()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        spend = await response.parse()
        assert_matches_type(SpendListTagsResponse, spend, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_tags(self, async_client: AsyncHanzo) -> None:
        async with async_client.spend.with_streaming_response.list_tags() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            spend = await response.parse()
            assert_matches_type(SpendListTagsResponse, spend, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .pass_through_endpoint import (
    PassThroughEndpointResource,
    AsyncPassThroughEndpointResource,
    PassThroughEndpointResourceWithRawResponse,
    AsyncPassThroughEndpointResourceWithRawResponse,
    PassThroughEndpointResourceWithStreamingResponse,
    AsyncPassThroughEndpointResourceWithStreamingResponse,
)

__all__ = ["ConfigResource", "AsyncConfigResource"]


class ConfigResource(SyncAPIResource):
    @cached_property
    def pass_through_endpoint(self) -> PassThroughEndpointResource:
        return PassThroughEndpointResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return ConfigResourceWithStreamingResponse(self)


class AsyncConfigResource(AsyncAPIResource):
    @cached_property
    def pass_through_endpoint(self) -> AsyncPassThroughEndpointResource:
        return AsyncPassThroughEndpointResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncConfigResourceWithStreamingResponse(self)


class ConfigResourceWithRawResponse:
    def __init__(self, config: ConfigResource) -> None:
        self._config = config

    @cached_property
    def pass_through_endpoint(self) -> PassThroughEndpointResourceWithRawResponse:
        return PassThroughEndpointResourceWithRawResponse(self._config.pass_through_endpoint)


class AsyncConfigResourceWithRawResponse:
    def __init__(self, config: AsyncConfigResource) -> None:
        self._config = config

    @cached_property
    def pass_through_endpoint(self) -> AsyncPassThroughEndpointResourceWithRawResponse:
        return AsyncPassThroughEndpointResourceWithRawResponse(self._config.pass_through_endpoint)


class ConfigResourceWithStreamingResponse:
    def __init__(self, config: ConfigResource) -> None:
        self._config = config

    @cached_property
    def pass_through_endpoint(self) -> PassThroughEndpointResourceWithStreamingResponse:
        return PassThroughEndpointResourceWithStreamingResponse(self._config.pass_through_endpoint)


class AsyncConfigResourceWithStreamingResponse:
    def __init__(self, config: AsyncConfigResource) -> None:
        self._config = config

    @cached_property
    def pass_through_endpoint(self) -> AsyncPassThroughEndpointResourceWithStreamingResponse:
        return AsyncPassThroughEndpointResourceWithStreamingResponse(self._config.pass_through_endpoint)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .organization_membership_table import OrganizationMembershipTable

__all__ = ["LiteLlmUserTable"]


class LiteLlmUserTable(BaseModel):
    user_id: str

    budget_duration: Optional[str] = None

    budget_reset_at: Optional[datetime] = None

    max_budget: Optional[float] = None

    metadata: Optional[object] = None

    api_model_max_budget: Optional[object] = FieldInfo(alias="model_max_budget", default=None)

    api_model_spend: Optional[object] = FieldInfo(alias="model_spend", default=None)

    models: Optional[List[object]] = None

    organization_memberships: Optional[List[OrganizationMembershipTable]] = None

    rpm_limit: Optional[int] = None

    spend: Optional[float] = None

    sso_user_id: Optional[str] = None

    teams: Optional[List[str]] = None

    tpm_limit: Optional[int] = None

    user_email: Optional[str] = None

    user_role: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import add_add_allowed_ip_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["AddResource", "AsyncAddResource"]


class AddResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AddResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AddResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AddResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AddResourceWithStreamingResponse(self)

    def add_allowed_ip(
        self,
        *,
        ip: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add Allowed Ip

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/add/allowed_ip",
            body=maybe_transform({"ip": ip}, add_add_allowed_ip_params.AddAddAllowedIPParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAddResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAddResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAddResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAddResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncAddResourceWithStreamingResponse(self)

    async def add_allowed_ip(
        self,
        *,
        ip: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add Allowed Ip

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/add/allowed_ip",
            body=await async_maybe_transform({"ip": ip}, add_add_allowed_ip_params.AddAddAllowedIPParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AddResourceWithRawResponse:
    def __init__(self, add: AddResource) -> None:
        self._add = add

        self.add_allowed_ip = to_raw_response_wrapper(
            add.add_allowed_ip,
        )


class AsyncAddResourceWithRawResponse:
    def __init__(self, add: AsyncAddResource) -> None:
        self._add = add

        self.add_allowed_ip = async_to_raw_response_wrapper(
            add.add_allowed_ip,
        )


class AddResourceWithStreamingResponse:
    def __init__(self, add: AddResource) -> None:
        self._add = add

        self.add_allowed_ip = to_streamed_response_wrapper(
            add.add_allowed_ip,
        )


class AsyncAddResourceWithStreamingResponse:
    def __init__(self, add: AsyncAddResource) -> None:
        self._add = add

        self.add_allowed_ip = async_to_streamed_response_wrapper(
            add.add_allowed_ip,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from .pass_through_endpoint import (
    PassThroughEndpointResource,
    AsyncPassThroughEndpointResource,
    PassThroughEndpointResourceWithRawResponse,
    AsyncPassThroughEndpointResourceWithRawResponse,
    PassThroughEndpointResourceWithStreamingResponse,
    AsyncPassThroughEndpointResourceWithStreamingResponse,
)

__all__ = [
    "PassThroughEndpointResource",
    "AsyncPassThroughEndpointResource",
    "PassThroughEndpointResourceWithRawResponse",
    "AsyncPassThroughEndpointResourceWithRawResponse",
    "PassThroughEndpointResourceWithStreamingResponse",
    "AsyncPassThroughEndpointResourceWithStreamingResponse",
    "ConfigResource",
    "AsyncConfigResource",
    "ConfigResourceWithRawResponse",
    "AsyncConfigResourceWithRawResponse",
    "ConfigResourceWithStreamingResponse",
    "AsyncConfigResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["JobRetrieveParams"]


class JobRetrieveParams(TypedDict, total=False):
    custom_llm_provider: Required[Literal["openai", "azure"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["SpendListLogsParams"]


class SpendListLogsParams(TypedDict, total=False):
    api_key: Optional[str]
    """Get spend logs based on api key"""

    end_date: Optional[str]
    """Time till which to view key spend"""

    request_id: Optional[str]
    """request_id to get spend logs for specific request_id.

    If none passed then pass spend logs for all requests
    """

    start_date: Optional[str]
    """Time from which to start viewing key spend"""

    user_id: Optional[str]
    """Get spend logs based on user_id"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional

import httpx

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from ...types import (
    UserRoles,
    organization_create_params,
    organization_delete_params,
    organization_update_params,
    organization_add_member_params,
    organization_delete_member_params,
    organization_update_member_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.user_roles import UserRoles
from ...types.organization_list_response import OrganizationListResponse
from ...types.organization_create_response import OrganizationCreateResponse
from ...types.organization_delete_response import OrganizationDeleteResponse
from ...types.organization_membership_table import OrganizationMembershipTable
from ...types.organization_table_with_members import OrganizationTableWithMembers
from ...types.organization_add_member_response import OrganizationAddMemberResponse

__all__ = ["OrganizationResource", "AsyncOrganizationResource"]


class OrganizationResource(SyncAPIResource):
    @cached_property
    def info(self) -> InfoResource:
        return InfoResource(self._client)

    @cached_property
    def with_raw_response(self) -> OrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return OrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return OrganizationResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        organization_alias: str,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        models: Iterable[object] | NotGiven = NOT_GIVEN,
        organization_id: Optional[str] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationCreateResponse:
        """
        Allow orgs to own teams

        Set org level budgets + model access.

        Only admins can create orgs.

        # Parameters

        - organization_alias: _str_ - The name of the organization.
        - models: _List_ - The models the organization has access to.
        - budget_id: _Optional[str]_ - The id for a budget (tpm/rpm/max budget) for the
          organization.

        ### IF NO BUDGET ID - CREATE ONE WITH THESE PARAMS

        - max_budget: _Optional[float]_ - Max budget for org
        - tpm_limit: _Optional[int]_ - Max tpm limit for org
        - rpm_limit: _Optional[int]_ - Max rpm limit for org
        - max_parallel_requests: _Optional[int]_ - [Not Implemented Yet] Max parallel
          requests for org
        - soft_budget: _Optional[float]_ - [Not Implemented Yet] Get a slack alert when
          this soft budget is reached. Don't block requests.
        - model_max_budget: _Optional[dict]_ - Max budget for a specific model
        - budget_duration: _Optional[str]_ - Frequency of reseting org budget
        - metadata: _Optional[dict]_ - Metadata for organization, store information for
          organization. Example metadata - {"extra_info": "some info"}
        - blocked: _bool_ - Flag indicating if the org is blocked or not - will stop all
          calls from keys with this org_id.
        - tags: _Optional[List[str]]_ - Tags for
          [tracking spend](https://litellm.vercel.app/docs/proxy/enterprise#tracking-spend-for-custom-tags)
          and/or doing
          [tag-based routing](https://litellm.vercel.app/docs/proxy/tag_routing).
        - organization_id: _Optional[str]_ - The organization id of the team. Default is
          None. Create via `/organization/new`.
        - model_aliases: Optional[dict] - Model aliases for the team.
          [Docs](https://docs.litellm.ai/docs/proxy/team_based_routing#create-team-with-model-alias)

        Case 1: Create new org **without** a budget_id

        ```bash
        curl --location 'http://0.0.0.0:4000/organization/new'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data '{
            "organization_alias": "my-secret-org",
            "models": ["model1", "model2"],
            "max_budget": 100
        }'


        ```

        Case 2: Create new org **with** a budget_id

        ```bash
        curl --location 'http://0.0.0.0:4000/organization/new'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data '{
            "organization_alias": "my-secret-org",
            "models": ["model1", "model2"],
            "budget_id": "428eeaa8-f3ac-4e85-a8fb-7dc8d7aa8689"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/new",
            body=maybe_transform(
                {
                    "organization_alias": organization_alias,
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "models": models,
                    "organization_id": organization_id,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                organization_create_params.OrganizationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationCreateResponse,
        )

    def update(
        self,
        *,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[List[str]] | NotGiven = NOT_GIVEN,
        organization_alias: Optional[str] | NotGiven = NOT_GIVEN,
        organization_id: Optional[str] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        updated_by: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationTableWithMembers:
        """
        Update an organization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/organization/update",
            body=maybe_transform(
                {
                    "budget_id": budget_id,
                    "metadata": metadata,
                    "models": models,
                    "organization_alias": organization_alias,
                    "organization_id": organization_id,
                    "spend": spend,
                    "updated_by": updated_by,
                },
                organization_update_params.OrganizationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationTableWithMembers,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationListResponse:
        """
        ```
        curl --location --request GET 'http://0.0.0.0:4000/organization/list'         --header 'Authorization: Bearer sk-1234'
        ```
        """
        return self._get(
            "/organization/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationListResponse,
        )

    def delete(
        self,
        *,
        organization_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationDeleteResponse:
        """
        Delete an organization

        # Parameters:

        - organization_ids: List[str] - The organization ids to delete.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/organization/delete",
            body=maybe_transform(
                {"organization_ids": organization_ids}, organization_delete_params.OrganizationDeleteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationDeleteResponse,
        )

    def add_member(
        self,
        *,
        member: organization_add_member_params.Member,
        organization_id: str,
        max_budget_in_organization: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationAddMemberResponse:
        """
        [BETA]

        Add new members (either via user_email or user_id) to an organization

        If user doesn't exist, new user row will also be added to User Table

        Only proxy_admin or org_admin of organization, allowed to access this endpoint.

        # Parameters:

        - organization_id: str (required)
        - member: Union[List[Member], Member] (required)
          - role: Literal[LitellmUserRoles] (required)
          - user_id: Optional[str]
          - user_email: Optional[str]

        Note: Either user_id or user_email must be provided for each member.

        Example:

        ```
        curl -X POST 'http://0.0.0.0:4000/organization/member_add'     -H 'Authorization: Bearer sk-1234'     -H 'Content-Type: application/json'     -d '{
            "organization_id": "45e3e396-ee08-4a61-a88e-16b3ce7e0849",
            "member": {
                "role": "internal_user",
                "user_id": "krrish247652@berri.ai"
            },
            "max_budget_in_organization": 100.0
        }'
        ```

        The following is executed in this function:

        1. Check if organization exists
        2. Creates a new Internal User if the user_id or user_email is not found in
           LiteLLM_UserTable
        3. Add Internal User to the `LiteLLM_OrganizationMembership` table

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/member_add",
            body=maybe_transform(
                {
                    "member": member,
                    "organization_id": organization_id,
                    "max_budget_in_organization": max_budget_in_organization,
                },
                organization_add_member_params.OrganizationAddMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationAddMemberResponse,
        )

    def delete_member(
        self,
        *,
        organization_id: str,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete a member from an organization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/organization/member_delete",
            body=maybe_transform(
                {
                    "organization_id": organization_id,
                    "user_email": user_email,
                    "user_id": user_id,
                },
                organization_delete_member_params.OrganizationDeleteMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update_member(
        self,
        *,
        organization_id: str,
        max_budget_in_organization: Optional[float] | NotGiven = NOT_GIVEN,
        role: Optional[UserRoles] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationMembershipTable:
        """
        Update a member's role in an organization

        Args:
          role: Admin Roles: PROXY_ADMIN: admin over the platform PROXY_ADMIN_VIEW_ONLY: can
              login, view all own keys, view all spend ORG_ADMIN: admin over a specific
              organization, can create teams, users only within their organization

              Internal User Roles: INTERNAL_USER: can login, view/create/delete their own
              keys, view their spend INTERNAL_USER_VIEW_ONLY: can login, view their own keys,
              view their own spend

              Team Roles: TEAM: used for JWT auth

              Customer Roles: CUSTOMER: External users -> these are customers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/organization/member_update",
            body=maybe_transform(
                {
                    "organization_id": organization_id,
                    "max_budget_in_organization": max_budget_in_organization,
                    "role": role,
                    "user_email": user_email,
                    "user_id": user_id,
                },
                organization_update_member_params.OrganizationUpdateMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationMembershipTable,
        )


class AsyncOrganizationResource(AsyncAPIResource):
    @cached_property
    def info(self) -> AsyncInfoResource:
        return AsyncInfoResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncOrganizationResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        organization_alias: str,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[object] | NotGiven = NOT_GIVEN,
        models: Iterable[object] | NotGiven = NOT_GIVEN,
        organization_id: Optional[str] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationCreateResponse:
        """
        Allow orgs to own teams

        Set org level budgets + model access.

        Only admins can create orgs.

        # Parameters

        - organization_alias: _str_ - The name of the organization.
        - models: _List_ - The models the organization has access to.
        - budget_id: _Optional[str]_ - The id for a budget (tpm/rpm/max budget) for the
          organization.

        ### IF NO BUDGET ID - CREATE ONE WITH THESE PARAMS

        - max_budget: _Optional[float]_ - Max budget for org
        - tpm_limit: _Optional[int]_ - Max tpm limit for org
        - rpm_limit: _Optional[int]_ - Max rpm limit for org
        - max_parallel_requests: _Optional[int]_ - [Not Implemented Yet] Max parallel
          requests for org
        - soft_budget: _Optional[float]_ - [Not Implemented Yet] Get a slack alert when
          this soft budget is reached. Don't block requests.
        - model_max_budget: _Optional[dict]_ - Max budget for a specific model
        - budget_duration: _Optional[str]_ - Frequency of reseting org budget
        - metadata: _Optional[dict]_ - Metadata for organization, store information for
          organization. Example metadata - {"extra_info": "some info"}
        - blocked: _bool_ - Flag indicating if the org is blocked or not - will stop all
          calls from keys with this org_id.
        - tags: _Optional[List[str]]_ - Tags for
          [tracking spend](https://litellm.vercel.app/docs/proxy/enterprise#tracking-spend-for-custom-tags)
          and/or doing
          [tag-based routing](https://litellm.vercel.app/docs/proxy/tag_routing).
        - organization_id: _Optional[str]_ - The organization id of the team. Default is
          None. Create via `/organization/new`.
        - model_aliases: Optional[dict] - Model aliases for the team.
          [Docs](https://docs.litellm.ai/docs/proxy/team_based_routing#create-team-with-model-alias)

        Case 1: Create new org **without** a budget_id

        ```bash
        curl --location 'http://0.0.0.0:4000/organization/new'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data '{
            "organization_alias": "my-secret-org",
            "models": ["model1", "model2"],
            "max_budget": 100
        }'


        ```

        Case 2: Create new org **with** a budget_id

        ```bash
        curl --location 'http://0.0.0.0:4000/organization/new'
        --header 'Authorization: Bearer sk-1234'
        --header 'Content-Type: application/json'
        --data '{
            "organization_alias": "my-secret-org",
            "models": ["model1", "model2"],
            "budget_id": "428eeaa8-f3ac-4e85-a8fb-7dc8d7aa8689"
        }'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/new",
            body=await async_maybe_transform(
                {
                    "organization_alias": organization_alias,
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "metadata": metadata,
                    "model_max_budget": model_max_budget,
                    "models": models,
                    "organization_id": organization_id,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                organization_create_params.OrganizationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationCreateResponse,
        )

    async def update(
        self,
        *,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Optional[object] | NotGiven = NOT_GIVEN,
        models: Optional[List[str]] | NotGiven = NOT_GIVEN,
        organization_alias: Optional[str] | NotGiven = NOT_GIVEN,
        organization_id: Optional[str] | NotGiven = NOT_GIVEN,
        spend: Optional[float] | NotGiven = NOT_GIVEN,
        updated_by: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationTableWithMembers:
        """
        Update an organization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/organization/update",
            body=await async_maybe_transform(
                {
                    "budget_id": budget_id,
                    "metadata": metadata,
                    "models": models,
                    "organization_alias": organization_alias,
                    "organization_id": organization_id,
                    "spend": spend,
                    "updated_by": updated_by,
                },
                organization_update_params.OrganizationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationTableWithMembers,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationListResponse:
        """
        ```
        curl --location --request GET 'http://0.0.0.0:4000/organization/list'         --header 'Authorization: Bearer sk-1234'
        ```
        """
        return await self._get(
            "/organization/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationListResponse,
        )

    async def delete(
        self,
        *,
        organization_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationDeleteResponse:
        """
        Delete an organization

        # Parameters:

        - organization_ids: List[str] - The organization ids to delete.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/organization/delete",
            body=await async_maybe_transform(
                {"organization_ids": organization_ids}, organization_delete_params.OrganizationDeleteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationDeleteResponse,
        )

    async def add_member(
        self,
        *,
        member: organization_add_member_params.Member,
        organization_id: str,
        max_budget_in_organization: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationAddMemberResponse:
        """
        [BETA]

        Add new members (either via user_email or user_id) to an organization

        If user doesn't exist, new user row will also be added to User Table

        Only proxy_admin or org_admin of organization, allowed to access this endpoint.

        # Parameters:

        - organization_id: str (required)
        - member: Union[List[Member], Member] (required)
          - role: Literal[LitellmUserRoles] (required)
          - user_id: Optional[str]
          - user_email: Optional[str]

        Note: Either user_id or user_email must be provided for each member.

        Example:

        ```
        curl -X POST 'http://0.0.0.0:4000/organization/member_add'     -H 'Authorization: Bearer sk-1234'     -H 'Content-Type: application/json'     -d '{
            "organization_id": "45e3e396-ee08-4a61-a88e-16b3ce7e0849",
            "member": {
                "role": "internal_user",
                "user_id": "krrish247652@berri.ai"
            },
            "max_budget_in_organization": 100.0
        }'
        ```

        The following is executed in this function:

        1. Check if organization exists
        2. Creates a new Internal User if the user_id or user_email is not found in
           LiteLLM_UserTable
        3. Add Internal User to the `LiteLLM_OrganizationMembership` table

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/member_add",
            body=await async_maybe_transform(
                {
                    "member": member,
                    "organization_id": organization_id,
                    "max_budget_in_organization": max_budget_in_organization,
                },
                organization_add_member_params.OrganizationAddMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationAddMemberResponse,
        )

    async def delete_member(
        self,
        *,
        organization_id: str,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete a member from an organization

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/organization/member_delete",
            body=await async_maybe_transform(
                {
                    "organization_id": organization_id,
                    "user_email": user_email,
                    "user_id": user_id,
                },
                organization_delete_member_params.OrganizationDeleteMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update_member(
        self,
        *,
        organization_id: str,
        max_budget_in_organization: Optional[float] | NotGiven = NOT_GIVEN,
        role: Optional[UserRoles] | NotGiven = NOT_GIVEN,
        user_email: Optional[str] | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationMembershipTable:
        """
        Update a member's role in an organization

        Args:
          role: Admin Roles: PROXY_ADMIN: admin over the platform PROXY_ADMIN_VIEW_ONLY: can
              login, view all own keys, view all spend ORG_ADMIN: admin over a specific
              organization, can create teams, users only within their organization

              Internal User Roles: INTERNAL_USER: can login, view/create/delete their own
              keys, view their spend INTERNAL_USER_VIEW_ONLY: can login, view their own keys,
              view their own spend

              Team Roles: TEAM: used for JWT auth

              Customer Roles: CUSTOMER: External users -> these are customers

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/organization/member_update",
            body=await async_maybe_transform(
                {
                    "organization_id": organization_id,
                    "max_budget_in_organization": max_budget_in_organization,
                    "role": role,
                    "user_email": user_email,
                    "user_id": user_id,
                },
                organization_update_member_params.OrganizationUpdateMemberParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationMembershipTable,
        )


class OrganizationResourceWithRawResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

        self.create = to_raw_response_wrapper(
            organization.create,
        )
        self.update = to_raw_response_wrapper(
            organization.update,
        )
        self.list = to_raw_response_wrapper(
            organization.list,
        )
        self.delete = to_raw_response_wrapper(
            organization.delete,
        )
        self.add_member = to_raw_response_wrapper(
            organization.add_member,
        )
        self.delete_member = to_raw_response_wrapper(
            organization.delete_member,
        )
        self.update_member = to_raw_response_wrapper(
            organization.update_member,
        )

    @cached_property
    def info(self) -> InfoResourceWithRawResponse:
        return InfoResourceWithRawResponse(self._organization.info)


class AsyncOrganizationResourceWithRawResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

        self.create = async_to_raw_response_wrapper(
            organization.create,
        )
        self.update = async_to_raw_response_wrapper(
            organization.update,
        )
        self.list = async_to_raw_response_wrapper(
            organization.list,
        )
        self.delete = async_to_raw_response_wrapper(
            organization.delete,
        )
        self.add_member = async_to_raw_response_wrapper(
            organization.add_member,
        )
        self.delete_member = async_to_raw_response_wrapper(
            organization.delete_member,
        )
        self.update_member = async_to_raw_response_wrapper(
            organization.update_member,
        )

    @cached_property
    def info(self) -> AsyncInfoResourceWithRawResponse:
        return AsyncInfoResourceWithRawResponse(self._organization.info)


class OrganizationResourceWithStreamingResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

        self.create = to_streamed_response_wrapper(
            organization.create,
        )
        self.update = to_streamed_response_wrapper(
            organization.update,
        )
        self.list = to_streamed_response_wrapper(
            organization.list,
        )
        self.delete = to_streamed_response_wrapper(
            organization.delete,
        )
        self.add_member = to_streamed_response_wrapper(
            organization.add_member,
        )
        self.delete_member = to_streamed_response_wrapper(
            organization.delete_member,
        )
        self.update_member = to_streamed_response_wrapper(
            organization.update_member,
        )

    @cached_property
    def info(self) -> InfoResourceWithStreamingResponse:
        return InfoResourceWithStreamingResponse(self._organization.info)


class AsyncOrganizationResourceWithStreamingResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

        self.create = async_to_streamed_response_wrapper(
            organization.create,
        )
        self.update = async_to_streamed_response_wrapper(
            organization.update,
        )
        self.list = async_to_streamed_response_wrapper(
            organization.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            organization.delete,
        )
        self.add_member = async_to_streamed_response_wrapper(
            organization.add_member,
        )
        self.delete_member = async_to_streamed_response_wrapper(
            organization.delete_member,
        )
        self.update_member = async_to_streamed_response_wrapper(
            organization.update_member,
        )

    @cached_property
    def info(self) -> AsyncInfoResourceWithStreamingResponse:
        return AsyncInfoResourceWithStreamingResponse(self._organization.info)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .spend_list_tags_params import SpendListTagsParams as SpendListTagsParams
from .spend_list_tags_response import SpendListTagsResponse as SpendListTagsResponse
from .spend_retrieve_report_params import SpendRetrieveReportParams as SpendRetrieveReportParams
from .spend_retrieve_report_response import SpendRetrieveReportResponse as SpendRetrieveReportResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CallbackAddParams"]


class CallbackAddParams(TypedDict, total=False):
    callback_name: Required[str]

    callback_vars: Required[Dict[str, str]]

    callback_type: Optional[Literal["success", "failure", "success_and_failure"]]

    litellm_changed_by: Annotated[str, PropertyInfo(alias="litellm-changed-by")]
    """
    The litellm-changed-by header enables tracking of actions performed by
    authorized users on behalf of other users, providing an audit trail for
    accountability
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TeamRetrieveInfoParams"]


class TeamRetrieveInfoParams(TypedDict, total=False):
    team_id: str
    """Team ID in the request parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBudget:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Hanzo) -> None:
        budget = client.budget.create()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Hanzo) -> None:
        budget = client.budget.create(
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Hanzo) -> None:
        budget = client.budget.update()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Hanzo) -> None:
        budget = client.budget.update(
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hanzo) -> None:
        budget = client.budget.list()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Hanzo) -> None:
        budget = client.budget.delete(
            id="id",
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_info(self, client: Hanzo) -> None:
        budget = client.budget.info(
            budgets=["string"],
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_info(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.info(
            budgets=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_info(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.info(
            budgets=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_settings(self, client: Hanzo) -> None:
        budget = client.budget.settings(
            budget_id="budget_id",
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_settings(self, client: Hanzo) -> None:
        response = client.budget.with_raw_response.settings(
            budget_id="budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_settings(self, client: Hanzo) -> None:
        with client.budget.with_streaming_response.settings(
            budget_id="budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBudget:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.create()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.create(
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.update()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.update(
            budget_duration="budget_duration",
            budget_id="budget_id",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.list()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.delete(
            id="id",
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_info(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.info(
            budgets=["string"],
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_info(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.info(
            budgets=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_info(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.info(
            budgets=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_settings(self, async_client: AsyncHanzo) -> None:
        budget = await async_client.budget.settings(
            budget_id="budget_id",
        )
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_settings(self, async_client: AsyncHanzo) -> None:
        response = await async_client.budget.with_raw_response.settings(
            budget_id="budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(object, budget, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_settings(self, async_client: AsyncHanzo) -> None:
        async with async_client.budget.with_streaming_response.settings(
            budget_id="budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(object, budget, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from .organization import (
    OrganizationResource,
    AsyncOrganizationResource,
    OrganizationResourceWithRawResponse,
    AsyncOrganizationResourceWithRawResponse,
    OrganizationResourceWithStreamingResponse,
    AsyncOrganizationResourceWithStreamingResponse,
)

__all__ = [
    "InfoResource",
    "AsyncInfoResource",
    "InfoResourceWithRawResponse",
    "AsyncInfoResourceWithRawResponse",
    "InfoResourceWithStreamingResponse",
    "AsyncInfoResourceWithStreamingResponse",
    "OrganizationResource",
    "AsyncOrganizationResource",
    "OrganizationResourceWithRawResponse",
    "AsyncOrganizationResourceWithRawResponse",
    "OrganizationResourceWithStreamingResponse",
    "AsyncOrganizationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options

__all__ = ["CancelResource", "AsyncCancelResource"]


class CancelResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CancelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return CancelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CancelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return CancelResourceWithStreamingResponse(self)

    def create(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Cancel a fine-tuning job.

        This is the equivalent of POST
        https://api.openai.com/v1/fine_tuning/jobs/{fine_tuning_job_id}/cancel

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `fine_tuning_job_id`: The ID of the fine-tuning job to cancel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._post(
            f"/v1/fine_tuning/jobs/{fine_tuning_job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncCancelResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCancelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCancelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCancelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncCancelResourceWithStreamingResponse(self)

    async def create(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Cancel a fine-tuning job.

        This is the equivalent of POST
        https://api.openai.com/v1/fine_tuning/jobs/{fine_tuning_job_id}/cancel

        Supported Query Params:

        - `custom_llm_provider`: Name of the LiteLLM provider
        - `fine_tuning_job_id`: The ID of the fine-tuning job to cancel.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return await self._post(
            f"/v1/fine_tuning/jobs/{fine_tuning_job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class CancelResourceWithRawResponse:
    def __init__(self, cancel: CancelResource) -> None:
        self._cancel = cancel

        self.create = to_raw_response_wrapper(
            cancel.create,
        )


class AsyncCancelResourceWithRawResponse:
    def __init__(self, cancel: AsyncCancelResource) -> None:
        self._cancel = cancel

        self.create = async_to_raw_response_wrapper(
            cancel.create,
        )


class CancelResourceWithStreamingResponse:
    def __init__(self, cancel: CancelResource) -> None:
        self._cancel = cancel

        self.create = to_streamed_response_wrapper(
            cancel.create,
        )


class AsyncCancelResourceWithStreamingResponse:
    def __init__(self, cancel: AsyncCancelResource) -> None:
        self._cancel = cancel

        self.create = async_to_streamed_response_wrapper(
            cancel.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .openai import (
    OpenAIResource,
    AsyncOpenAIResource,
    OpenAIResourceWithRawResponse,
    AsyncOpenAIResourceWithRawResponse,
    OpenAIResourceWithStreamingResponse,
    AsyncOpenAIResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)

__all__ = [
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
    "OpenAIResource",
    "AsyncOpenAIResource",
    "OpenAIResourceWithRawResponse",
    "AsyncOpenAIResourceWithRawResponse",
    "OpenAIResourceWithStreamingResponse",
    "AsyncOpenAIResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["AssistantsResource", "AsyncAssistantsResource"]


class AssistantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AssistantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AssistantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssistantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AssistantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Create assistant

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/createAssistant
        """
        return self._post(
            "/v1/assistants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Returns a list of assistants.

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/listAssistants
        """
        return self._get(
            "/v1/assistants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete assistant

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/createAssistant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._delete(
            f"/v1/assistants/{assistant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAssistantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAssistantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAssistantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssistantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncAssistantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Create assistant

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/createAssistant
        """
        return await self._post(
            "/v1/assistants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Returns a list of assistants.

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/listAssistants
        """
        return await self._get(
            "/v1/assistants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete assistant

        API Reference docs -
        https://platform.openai.com/docs/api-reference/assistants/createAssistant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._delete(
            f"/v1/assistants/{assistant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AssistantsResourceWithRawResponse:
    def __init__(self, assistants: AssistantsResource) -> None:
        self._assistants = assistants

        self.create = to_raw_response_wrapper(
            assistants.create,
        )
        self.list = to_raw_response_wrapper(
            assistants.list,
        )
        self.delete = to_raw_response_wrapper(
            assistants.delete,
        )


class AsyncAssistantsResourceWithRawResponse:
    def __init__(self, assistants: AsyncAssistantsResource) -> None:
        self._assistants = assistants

        self.create = async_to_raw_response_wrapper(
            assistants.create,
        )
        self.list = async_to_raw_response_wrapper(
            assistants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            assistants.delete,
        )


class AssistantsResourceWithStreamingResponse:
    def __init__(self, assistants: AssistantsResource) -> None:
        self._assistants = assistants

        self.create = to_streamed_response_wrapper(
            assistants.create,
        )
        self.list = to_streamed_response_wrapper(
            assistants.list,
        )
        self.delete = to_streamed_response_wrapper(
            assistants.delete,
        )


class AsyncAssistantsResourceWithStreamingResponse:
    def __init__(self, assistants: AsyncAssistantsResource) -> None:
        self._assistants = assistants

        self.create = async_to_streamed_response_wrapper(
            assistants.create,
        )
        self.list = async_to_streamed_response_wrapper(
            assistants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            assistants.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options

__all__ = ["ContentResource", "AsyncContentResource"]


class ContentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ContentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return ContentResourceWithStreamingResponse(self)

    def retrieve(
        self,
        file_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Returns information about a specific file.

        that can be used across - Assistants
        API, Batch API This is the equivalent of GET
        https://api.openai.com/v1/files/{file_id}/content

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/files/retrieve-contents

        Example Curl

        ```
        curl http://localhost:4000/v1/files/file-abc123/content         -H "Authorization: Bearer sk-1234"

        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return self._get(
            f"/{provider}/v1/files/{file_id}/content",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncContentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncContentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncContentResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        file_id: str,
        *,
        provider: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Returns information about a specific file.

        that can be used across - Assistants
        API, Batch API This is the equivalent of GET
        https://api.openai.com/v1/files/{file_id}/content

        Supports Identical Params as:
        https://platform.openai.com/docs/api-reference/files/retrieve-contents

        Example Curl

        ```
        curl http://localhost:4000/v1/files/file-abc123/content         -H "Authorization: Bearer sk-1234"

        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not provider:
            raise ValueError(f"Expected a non-empty value for `provider` but received {provider!r}")
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return await self._get(
            f"/{provider}/v1/files/{file_id}/content",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ContentResourceWithRawResponse:
    def __init__(self, content: ContentResource) -> None:
        self._content = content

        self.retrieve = to_raw_response_wrapper(
            content.retrieve,
        )


class AsyncContentResourceWithRawResponse:
    def __init__(self, content: AsyncContentResource) -> None:
        self._content = content

        self.retrieve = async_to_raw_response_wrapper(
            content.retrieve,
        )


class ContentResourceWithStreamingResponse:
    def __init__(self, content: ContentResource) -> None:
        self._content = content

        self.retrieve = to_streamed_response_wrapper(
            content.retrieve,
        )


class AsyncContentResourceWithStreamingResponse:
    def __init__(self, content: AsyncContentResource) -> None:
        self._content = content

        self.retrieve = async_to_streamed_response_wrapper(
            content.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["OrganizationCreateParams"]


class OrganizationCreateParams(TypedDict, total=False):
    organization_alias: Required[str]

    budget_duration: Optional[str]

    budget_id: Optional[str]

    max_budget: Optional[float]

    max_parallel_requests: Optional[int]

    metadata: Optional[object]

    model_max_budget: Optional[object]

    models: Iterable[object]

    organization_id: Optional[str]

    rpm_limit: Optional[int]

    soft_budget: Optional[float]

    tpm_limit: Optional[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .model_add_params import ModelAddParams as ModelAddParams
from .callback_add_params import CallbackAddParams as CallbackAddParams
from .model_remove_params import ModelRemoveParams as ModelRemoveParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional

import httpx

from ..types import (
    budget_info_params,
    budget_create_params,
    budget_delete_params,
    budget_update_params,
    budget_settings_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["BudgetResource", "AsyncBudgetResource"]


class BudgetResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BudgetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return BudgetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BudgetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return BudgetResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, budget_create_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create a new budget object.

        Can apply this to teams, orgs, end-users, keys.

        Parameters:

        - budget_duration: Optional[str] - Budget reset period ("30d", "1h", etc.)
        - budget_id: Optional[str] - The id of the budget. If not provided, a new id
          will be generated.
        - max_budget: Optional[float] - The max budget for the budget.
        - soft_budget: Optional[float] - The soft budget for the budget.
        - max_parallel_requests: Optional[int] - The max number of parallel requests for
          the budget.
        - tpm_limit: Optional[int] - The tokens per minute limit for the budget.
        - rpm_limit: Optional[int] - The requests per minute limit for the budget.
        - model_max_budget: Optional[dict] - Specify max budget for a given model.
          Example: {"openai/gpt-4o-mini": {"max_budget": 100.0, "budget_duration": "1d",
          "tpm_limit": 100000, "rpm_limit": 100000}}

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          budget_id: The unique budget id.

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/budget/new",
            body=maybe_transform(
                {
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                budget_create_params.BudgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update(
        self,
        *,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, budget_update_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Update an existing budget object.

        Parameters:

        - budget_duration: Optional[str] - Budget reset period ("30d", "1h", etc.)
        - budget_id: Optional[str] - The id of the budget. If not provided, a new id
          will be generated.
        - max_budget: Optional[float] - The max budget for the budget.
        - soft_budget: Optional[float] - The soft budget for the budget.
        - max_parallel_requests: Optional[int] - The max number of parallel requests for
          the budget.
        - tpm_limit: Optional[int] - The tokens per minute limit for the budget.
        - rpm_limit: Optional[int] - The requests per minute limit for the budget.
        - model_max_budget: Optional[dict] - Specify max budget for a given model.
          Example: {"openai/gpt-4o-mini": {"max_budget": 100.0, "budget_duration": "1d",
          "tpm_limit": 100000, "rpm_limit": 100000}}

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          budget_id: The unique budget id.

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/budget/update",
            body=maybe_transform(
                {
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                budget_update_params.BudgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """List all the created budgets in proxy db. Used on Admin UI."""
        return self._get(
            "/budget/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete budget

        Parameters:

        - id: str - The budget id to delete

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/budget/delete",
            body=maybe_transform({"id": id}, budget_delete_params.BudgetDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def info(
        self,
        *,
        budgets: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get the budget id specific information

        Parameters:

        - budgets: List[str] - The list of budget ids to get information for

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/budget/info",
            body=maybe_transform({"budgets": budgets}, budget_info_params.BudgetInfoParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def settings(
        self,
        *,
        budget_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get list of configurable params + current value for a budget item + description
        of each field

        Used on Admin UI.

        Query Parameters:

        - budget_id: str - The budget id to get information for

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/budget/settings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"budget_id": budget_id}, budget_settings_params.BudgetSettingsParams),
            ),
            cast_to=object,
        )


class AsyncBudgetResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBudgetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBudgetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBudgetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncBudgetResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, budget_create_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create a new budget object.

        Can apply this to teams, orgs, end-users, keys.

        Parameters:

        - budget_duration: Optional[str] - Budget reset period ("30d", "1h", etc.)
        - budget_id: Optional[str] - The id of the budget. If not provided, a new id
          will be generated.
        - max_budget: Optional[float] - The max budget for the budget.
        - soft_budget: Optional[float] - The soft budget for the budget.
        - max_parallel_requests: Optional[int] - The max number of parallel requests for
          the budget.
        - tpm_limit: Optional[int] - The tokens per minute limit for the budget.
        - rpm_limit: Optional[int] - The requests per minute limit for the budget.
        - model_max_budget: Optional[dict] - Specify max budget for a given model.
          Example: {"openai/gpt-4o-mini": {"max_budget": 100.0, "budget_duration": "1d",
          "tpm_limit": 100000, "rpm_limit": 100000}}

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          budget_id: The unique budget id.

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/budget/new",
            body=await async_maybe_transform(
                {
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                budget_create_params.BudgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update(
        self,
        *,
        budget_duration: Optional[str] | NotGiven = NOT_GIVEN,
        budget_id: Optional[str] | NotGiven = NOT_GIVEN,
        max_budget: Optional[float] | NotGiven = NOT_GIVEN,
        max_parallel_requests: Optional[int] | NotGiven = NOT_GIVEN,
        model_max_budget: Optional[Dict[str, budget_update_params.ModelMaxBudget]] | NotGiven = NOT_GIVEN,
        rpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        soft_budget: Optional[float] | NotGiven = NOT_GIVEN,
        tpm_limit: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Update an existing budget object.

        Parameters:

        - budget_duration: Optional[str] - Budget reset period ("30d", "1h", etc.)
        - budget_id: Optional[str] - The id of the budget. If not provided, a new id
          will be generated.
        - max_budget: Optional[float] - The max budget for the budget.
        - soft_budget: Optional[float] - The soft budget for the budget.
        - max_parallel_requests: Optional[int] - The max number of parallel requests for
          the budget.
        - tpm_limit: Optional[int] - The tokens per minute limit for the budget.
        - rpm_limit: Optional[int] - The requests per minute limit for the budget.
        - model_max_budget: Optional[dict] - Specify max budget for a given model.
          Example: {"openai/gpt-4o-mini": {"max_budget": 100.0, "budget_duration": "1d",
          "tpm_limit": 100000, "rpm_limit": 100000}}

        Args:
          budget_duration: Max duration budget should be set for (e.g. '1hr', '1d', '28d')

          budget_id: The unique budget id.

          max_budget: Requests will fail if this budget (in USD) is exceeded.

          max_parallel_requests: Max concurrent requests allowed for this budget id.

          model_max_budget: Max budget for each model (e.g. {'gpt-4o': {'max_budget': '0.0000001',
              'budget_duration': '1d', 'tpm_limit': 1000, 'rpm_limit': 1000}})

          rpm_limit: Max requests per minute, allowed for this budget id.

          soft_budget: Requests will NOT fail if this is exceeded. Will fire alerting though.

          tpm_limit: Max tokens per minute, allowed for this budget id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/budget/update",
            body=await async_maybe_transform(
                {
                    "budget_duration": budget_duration,
                    "budget_id": budget_id,
                    "max_budget": max_budget,
                    "max_parallel_requests": max_parallel_requests,
                    "model_max_budget": model_max_budget,
                    "rpm_limit": rpm_limit,
                    "soft_budget": soft_budget,
                    "tpm_limit": tpm_limit,
                },
                budget_update_params.BudgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """List all the created budgets in proxy db. Used on Admin UI."""
        return await self._get(
            "/budget/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Delete budget

        Parameters:

        - id: str - The budget id to delete

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/budget/delete",
            body=await async_maybe_transform({"id": id}, budget_delete_params.BudgetDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def info(
        self,
        *,
        budgets: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get the budget id specific information

        Parameters:

        - budgets: List[str] - The list of budget ids to get information for

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/budget/info",
            body=await async_maybe_transform({"budgets": budgets}, budget_info_params.BudgetInfoParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def settings(
        self,
        *,
        budget_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Get list of configurable params + current value for a budget item + description
        of each field

        Used on Admin UI.

        Query Parameters:

        - budget_id: str - The budget id to get information for

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/budget/settings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"budget_id": budget_id}, budget_settings_params.BudgetSettingsParams
                ),
            ),
            cast_to=object,
        )


class BudgetResourceWithRawResponse:
    def __init__(self, budget: BudgetResource) -> None:
        self._budget = budget

        self.create = to_raw_response_wrapper(
            budget.create,
        )
        self.update = to_raw_response_wrapper(
            budget.update,
        )
        self.list = to_raw_response_wrapper(
            budget.list,
        )
        self.delete = to_raw_response_wrapper(
            budget.delete,
        )
        self.info = to_raw_response_wrapper(
            budget.info,
        )
        self.settings = to_raw_response_wrapper(
            budget.settings,
        )


class AsyncBudgetResourceWithRawResponse:
    def __init__(self, budget: AsyncBudgetResource) -> None:
        self._budget = budget

        self.create = async_to_raw_response_wrapper(
            budget.create,
        )
        self.update = async_to_raw_response_wrapper(
            budget.update,
        )
        self.list = async_to_raw_response_wrapper(
            budget.list,
        )
        self.delete = async_to_raw_response_wrapper(
            budget.delete,
        )
        self.info = async_to_raw_response_wrapper(
            budget.info,
        )
        self.settings = async_to_raw_response_wrapper(
            budget.settings,
        )


class BudgetResourceWithStreamingResponse:
    def __init__(self, budget: BudgetResource) -> None:
        self._budget = budget

        self.create = to_streamed_response_wrapper(
            budget.create,
        )
        self.update = to_streamed_response_wrapper(
            budget.update,
        )
        self.list = to_streamed_response_wrapper(
            budget.list,
        )
        self.delete = to_streamed_response_wrapper(
            budget.delete,
        )
        self.info = to_streamed_response_wrapper(
            budget.info,
        )
        self.settings = to_streamed_response_wrapper(
            budget.settings,
        )


class AsyncBudgetResourceWithStreamingResponse:
    def __init__(self, budget: AsyncBudgetResource) -> None:
        self._budget = budget

        self.create = async_to_streamed_response_wrapper(
            budget.create,
        )
        self.update = async_to_streamed_response_wrapper(
            budget.update,
        )
        self.list = async_to_streamed_response_wrapper(
            budget.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            budget.delete,
        )
        self.info = async_to_streamed_response_wrapper(
            budget.info,
        )
        self.settings = async_to_streamed_response_wrapper(
            budget.settings,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Member"]


class Member(BaseModel):
    role: Literal["admin", "user"]

    user_email: Optional[str] = None

    user_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .transcription_create_params import TranscriptionCreateParams as TranscriptionCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["UserRoles"]

UserRoles: TypeAlias = Literal[
    "proxy_admin", "proxy_admin_viewer", "org_admin", "internal_user", "internal_user_viewer", "team", "customer"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TeamUpdateMemberParams"]


class TeamUpdateMemberParams(TypedDict, total=False):
    team_id: Required[str]

    max_budget_in_team: Optional[float]

    role: Optional[Literal["admin", "user"]]

    user_email: Optional[str]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["BudgetCreateParams", "ModelMaxBudget"]


class BudgetCreateParams(TypedDict, total=False):
    budget_duration: Optional[str]
    """Max duration budget should be set for (e.g. '1hr', '1d', '28d')"""

    budget_id: Optional[str]
    """The unique budget id."""

    max_budget: Optional[float]
    """Requests will fail if this budget (in USD) is exceeded."""

    max_parallel_requests: Optional[int]
    """Max concurrent requests allowed for this budget id."""

    model_max_budget: Optional[Dict[str, ModelMaxBudget]]
    """Max budget for each model (e.g.

    {'gpt-4o': {'max_budget': '0.0000001', 'budget_duration': '1d', 'tpm_limit':
    1000, 'rpm_limit': 1000}})
    """

    rpm_limit: Optional[int]
    """Max requests per minute, allowed for this budget id."""

    soft_budget: Optional[float]
    """Requests will NOT fail if this is exceeded. Will fire alerting though."""

    tpm_limit: Optional[int]
    """Max tokens per minute, allowed for this budget id."""


class ModelMaxBudget(TypedDict, total=False):
    budget_duration: Optional[str]

    max_budget: Optional[float]

    rpm_limit: Optional[int]

    tpm_limit: Optional[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .key import (
    KeyResource,
    AsyncKeyResource,
    KeyResourceWithRawResponse,
    AsyncKeyResourceWithRawResponse,
    KeyResourceWithStreamingResponse,
    AsyncKeyResourceWithStreamingResponse,
)

__all__ = [
    "KeyResource",
    "AsyncKeyResource",
    "KeyResourceWithRawResponse",
    "AsyncKeyResourceWithRawResponse",
    "KeyResourceWithStreamingResponse",
    "AsyncKeyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from .threads import (
    ThreadsResource,
    AsyncThreadsResource,
    ThreadsResourceWithRawResponse,
    AsyncThreadsResourceWithRawResponse,
    ThreadsResourceWithStreamingResponse,
    AsyncThreadsResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "RunsResource",
    "AsyncRunsResource",
    "RunsResourceWithRawResponse",
    "AsyncRunsResourceWithRawResponse",
    "RunsResourceWithStreamingResponse",
    "AsyncRunsResourceWithStreamingResponse",
    "ThreadsResource",
    "AsyncThreadsResource",
    "ThreadsResourceWithRawResponse",
    "AsyncThreadsResourceWithRawResponse",
    "ThreadsResourceWithStreamingResponse",
    "AsyncThreadsResourceWithStreamingResponse",
]
